/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2020-2021 Dmitriy Gorbunov (dmitriy.goto@gmail.com)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import SwiftUI
import Common

struct BottomNavigationView: View {
    
    private let pm: BottomNavigationPm
    
    @ObservedObject
    private var currentPm: ObservableState<PresentationModel> // TODO bind tab selection
    
    init(pm: BottomNavigationPm) {
        self.pm = pm
        self.currentPm = ObservableState(pm.navigator.currentState)
        
    }
    
    var body: some View {
        NavigationView {
            TabView {
                ForEach(pm.navigator.values, id: \.self) { itemPm in
                    if (itemPm is TabPm) {
                        let tabPm = itemPm as! TabPm
                        TabContainerView(pm: tabPm)
                            .tabItem {
                                Image(systemName: "star.fill")
                                Text(tabPm.tabTitle)
                            }
                            .tag(tabPm.tag)
                            .onAppear {
                                pm.navigator.setCurrent(pm: tabPm)
                            }
                    }
                }
            }
            .navigationTitle("Bottom Navigation")
            .navigationBarTitleDisplayMode(.inline)
            .navigationBarItems(leading: Button(action : {
                pm.back()
            }){
                Image(systemName: "arrow.left")
            })
        }
    }
}

struct BottomNavigationView_Previews: PreviewProvider {
    static var previews: some View {
        BottomNavigationView(pm: Stubs.init().bottomBarPm)
    }
}
